/*
 * PairwiseAligner.cpp
 *
 *  Created on: Mar 11, 2010
 *      Author: kohany
 */

#include "PairwiseAligner.h"

extern "C" void setSWSSE2Matrix(const signed char *mat, const short *char2index, const char *index2char, short alphabethSize);

PairwiseAligner::PairwiseAligner() :
#ifdef __SSE2__
   alignmentMethod(SWSSE2),
#else
   alignmentMethod(MONARDO),
#endif
   gapIni(0),
   gapExt(0) {
}

PairwiseAligner::~PairwiseAligner() {
   // TODO Auto-generated destructor stub
}

void PairwiseAligner::setAlignmentMatrix(const char* matfile) {
   ScoreMatrix alnMatrix;
   alnMatrix.parse(matfile);
   setAlignmentMatrix(alnMatrix);
#ifndef _OPEN_SOURCE_
   setSWATMAT(matfile);
#endif
}

void PairwiseAligner::setAlignmentMatrix(const ScoreMatrix& mat) {
   signed char c_mat[mat.getAlphabethSize()];
   for(int i = 0; i < mat.getAlphabethSize(); i++) {
      c_mat[i] = (signed char)mat.getMatrix()[i];
   }
#ifdef __SSE2__
   setSWSSE2Matrix(c_mat, mat.getChar2Index(), mat.getIndex2Char(), mat.getAlphabethSize());
#endif
#ifndef _OPEN_SOURCE_
      //setSWATMAT("/usr/local/lib/mat/BLOSUM62");
#endif
}

double PairwiseAligner::align(BioAlignment& aln) const {
   switch (getAlignmentMethod()) {
   case SWSSE2:
      aln.setScore(align2seq_swsse2(aln.getCon(), aln.getSeq(), gapIni, gapExt));
      break;
#ifndef _OPEN_SOURCE_
   case PGSWAT:
      aln.setScore(align2seq_pgswat(aln.getCon(), aln.getSeq(), gapIni, gapExt));
      break;
#endif
   case MONARDO:
   default:
      aln.setScore(align2seq_monardo(aln.getCon(), aln.getSeq()));
   }
   return aln.getScore();
}
