/*
 * ScoreMatrix.h
 *
 *  Created on: Aug 13, 2009
 *      Author: kohany
 */

#ifndef SCOREMATRIX_H_
#define SCOREMATRIX_H_

#include <string>
using namespace std;

class ScoreMatrix
{
   private:
      double *matrix;
      short char2index[255];
      char index2char[255];
      unsigned short alphabethSize;
      double gapIniPenalty;
      double gapExtPenalty;
      double gapMatchScore; // Used to score GAP-to-GAP pair.
      string& trim(string& s, char c = ' ');
   public:
      static const char ALNMATRIX[];
      static const char CONMATRIX[];
      static const char GAPCHAR;
      static const char STOPCODON;
      static const char LPADDING;
      static const char RPADDING;
      ScoreMatrix();
      ScoreMatrix(const ScoreMatrix& other);
      ScoreMatrix(const char* file);
      virtual ~ScoreMatrix();
      ScoreMatrix& assign(const ScoreMatrix& other);
      ScoreMatrix& operator = (const ScoreMatrix& other);
      bool loadMatrixFromString(const char* defaultMatrix = ScoreMatrix::ALNMATRIX);
      bool parse(const char* file);
      bool parse(istream& is);
      void print(ostream& os) const;
      double getScore(char c1, char c2) const;
      string getAlphabeth() const;
      double getMaxScore() const;
      double getGapIniPenalty() const
      {
         return gapIniPenalty;
      }
      double getGapExtPenalty() const
      {
         return gapExtPenalty;
      }
      double getGapMatchScore() const {
         return gapMatchScore;
      }
      void setGapIniPenalty(double penalty)
      {
         gapIniPenalty = penalty;
      }
      void setGapExtPenalty(double penalty)
      {
         gapExtPenalty = penalty;
      }
      void setGapMatchScore(double score) {
         gapMatchScore = score;
      }

      const double* getMatrix() const {
         return matrix;
      }
      const short* getChar2Index() const {
         return char2index;
      }
      const char* getIndex2Char() const {
         return index2char;
      }
      short getAlphabethSize() const {
         return alphabethSize;
      }
};

#endif /* SCOREMATRIX_H_ */
