/***************************************************************************
 main.cpp  -  description
 -------------------
 begin                : Fri Jan 17 17:59:14 UTC 2003
 copyright            : (C) 2003 by Oleksiy Kohany
 email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>
#include <assert.h>

#include "BioSequence.h"
#include "BioAlignment.h"
#include "PairwiseAligner.h"
#include "IOWrapper.h"
#include "ThreadPool.h"

#include <boost/program_options.hpp>
namespace po = boost::program_options;

using namespace std;

bool DEBUG = 0;

const AlnStat getCDSStat(const string s1, const string s2, int pos, int offset = 0) {
   int iSize = min(s1.size(), s2.size());
   BioAlignment aln;
   aln.getSeq().setName("s1");
   aln.getCon().setName("s2");

   for (int i = 0, p = -1; i < iSize; i++) {
      if (isalpha(s1[i])) {
         p++;
         if ((pos + p) % 3 == offset % 3) {
            aln.getSeq().append(s1[i]);
            aln.getCon().append(s2[i]);
         }
      }
   }

   if (DEBUG) {
      aln.getStat().print(stderr);
      aln.print(stderr);
   }

   return aln.getStat();
}
/** ======================================================== */
const AlnStat getCDSStatX(const string s1, const string s2, int pos, int offset = 0) {
   int iSize = min(s1.size(), s2.size());
   BioAlignment aln;
   aln.getSeq().setName("s1");
   aln.getCon().setName("s2");
   string cd1, cd2;
   int i = 0;
   while ((pos + i) % 3) {
      i++;
   }
   for (; i < (iSize - 1); i++) {
      if (isalpha(s1[i])) {
         cd1.append(1, (char) toupper(s1[i]));
         cd2.append(1, (char) toupper(s2[i]));
      }
      if (cd1.size() >= 3) {
         if (cd1[0] == 'A' && cd1[1] == 'C' || cd1[0] == 'C' && (cd1[1] == 'T' || cd1[1] == 'C' || cd1[1] == 'G') || cd1[0] == 'G' && (cd1[1] == 'G' || cd1[1] == 'C' || cd1[1] == 'T') || cd1[0]
               == 'T' && (cd1[1] == 'C')) {
            aln.getSeq().append(cd1[offset % 3]);
            aln.getCon().append(cd2[offset % 3]);
         }
         cd1.assign("");
         cd2.assign("");
      }
   }
   if (cd1.size() >= 2) {
      if (cd1[0] == 'A' && cd1[1] == 'C' || cd1[0] == 'C' && (cd1[1] == 'T' || cd1[1] == 'C' || cd1[1] == 'G') || cd1[0] == 'G' && (cd1[1] == 'G' || cd1[1] == 'C' || cd1[1] == 'T') || cd1[0] == 'T'
            && (cd1[1] == 'C')) {
         aln.getSeq().append(cd1[offset % 3]);
         aln.getCon().append(cd2[offset % 3]);
      }
   }
   if (DEBUG) {
      aln.getStat().print(stderr);
      aln.print(stderr);
   }
   return aln.getStat();
}
//===========================================================
class worker_params {
public:
   worker_params(istream& ifile1, istream& ifile2, ostream& ofile, PairwiseAligner& aligner, int offset, bool codonSubset, int edgeWidth):
      ifile1(ifile1),
      ifile2(ifile2),
      ofile(ofile),
      aligner(aligner),
      offset(offset),
      codonSubset(codonSubset),
      edgeWidth(edgeWidth)
      {
      assert(pthread_mutex_init(&align_mutex, NULL) == 0);
      assert(pthread_mutex_init(&read_mutex, NULL) == 0);
      assert(pthread_mutex_init(&write_mutex, NULL) == 0);
   }
   ~worker_params() {
      pthread_mutex_destroy(&align_mutex);
      pthread_mutex_destroy(&read_mutex);
      pthread_mutex_destroy(&write_mutex);
   }
   istream& ifile1;
   istream& ifile2;
   ostream& ofile;
   PairwiseAligner& aligner;
   int offset;
   bool codonSubset;
   int edgeWidth;
   pthread_mutex_t align_mutex;
   pthread_mutex_t read_mutex;
   pthread_mutex_t write_mutex;
};
//===========================================================
void *thread_func(void *args) {
   static int threadCount = 0;
   int threadId;
   worker_params *params = (worker_params*) args;
   BioAlignment aln;

   pthread_mutex_lock(&(params->read_mutex));
   threadId = threadCount++;
   pthread_mutex_unlock(&(params->read_mutex));

   while (pthread_mutex_lock(&(params->read_mutex)) == 0 && aln.getSeq().read(params->ifile1) >= 0 && aln.getCon().read(params->ifile2) >= 0) {
      pthread_mutex_unlock(&(params->read_mutex));

      aln.getCon().toupper();
      aln.getSeq().toupper();

      if (params->aligner.align(aln)) {
         if (params->edgeWidth > 0) {
            aln.sharp(params->edgeWidth);
            if (aln.getScore() == 0) {
               continue;
            }
         }
         AlnStat alnstat;
         if (params->offset) {
            int offset = params->offset - 1;
            if (params->codonSubset) {
               alnstat = getCDSStatX(aln.getCon(), aln.getSeq(), aln.getCon().getFrom() - 1, offset);
            } else {
               alnstat = getCDSStat(aln.getCon(), aln.getSeq(), aln.getCon().getFrom() - 1, offset);
            }
         }
         assert(pthread_mutex_lock(&(params->write_mutex)) == 0);
         if (params->offset) {
            params->ofile << aln.getCon().getName() << "\t+" << params->offset << "\t" << aln.getSeq().getName() << "\t";
            alnstat.print(params->ofile);
         }
         aln.print(params->ofile);
         pthread_mutex_unlock(&(params->write_mutex));
      } else {
         cerr << "WARNING: No alignment:\n";
         aln.getCon().printFASTA(cerr);
         aln.getSeq().printFASTA(cerr);
      }
   }
   pthread_mutex_unlock(&(params->read_mutex));
}

/** ======================================================== */
void usage(char* exec, po::options_description od) {
   if (!exec)
      return;
   if (strrchr(exec, '/'))
      exec = strrchr(exec, '/') + 1;
   cerr << "NAME\n\t" << exec << " - Aligns all seqeunces in first input file against corresponding\n";
   cerr << "\tsequences in second file, and optionally collects and displays additional\n";
   cerr << "\tinformation for given position in each codon.\n";

   cerr << "SYNOPSYS\n\t" << exec << " infile1(-) infile2 [outfile] [options]\n";
   cerr << od;
   cerr << "\nAUTHOR\n\tOleksiy Kohany, kohany@girinst.org\n";
#ifdef COPYRIGHT

   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}

/** ======================================================== */
void showVersion(const char* exec) {
   if (!exec)
      return;
   if (strrchr(exec, '/'))
      exec = strrchr(exec, '/') + 1;
   cerr << exec;
#ifdef VERSION

   cerr << "\tversion " << VERSION;
#endif

   cerr << endl;
}

/** ======================================================== */
int main(int argc, char *argv[]) {
   string infile1;
   string infile2;
   string outfile;

   PairwiseAligner aligner;
   int iGI = 0;
   int iGX = 0;
   string mask;

   int sharp = 0;
   int offset = 0;
   bool codonSubset = false;
   bool useThreads = true;

   po::options_description generic("GENERIC OPTIONS");
   po::options_description settings("SETTINGS");
   po::options_description alignment("ALIGNMENT OPTIONS");
   po::options_description backend("BACKEND OPTIONS");
   po::options_description hidden("HIDDEN OPTIONS");

   generic.add_options()
   ("version,v", "print version string.")
   ("help", "produce help message.")
   //("debug", "will show debugging messages on execution process.")
   ;

   settings.add_options()
   ("protein,p", "will assume that sequences are proteins.")
   ("msk", po::value<string>(&mask), "pattern that should not be included in calculating similarity, if found in bottom sequence.")
   ("pos", po::value<int>(&offset), "codon position (1-3). This activates display of additional information for a given position in each codon.")
   ("sharp", po::value<int>(&sharp), "alignment will be truncated on both ends until perfect alignments of given length are found on both ends of the alignment")
   ("except,x", "when used with --pos option the program will analyze only ACN GGN GCN CTN CCN TCN CGN GTN codons")
   ;

   alignment.add_options()
#ifndef _OPEN_SOURCE_
   ("pgsw", "uses Phil Green's implementation of SWAT algorithm. Default matrix - xrepmask, gi=30, gx=6")
#endif
#ifdef HAVE_SSE2
   ("sse2", "uses modified implementation of Michael Farrar's striped SWAT algorithm for SSE2 extension of x86 CPUs. This is the default alignment method. Default matrix - xrepmask/3, gi=-10, gx=-2. Limitations: Maximum score can not accede 65535 so keep alignment matrix's scores low. With the default nucleotide matrix you can align over 21KB.")
#endif
   ("mon", "use Monardo alignment algorithm. It has logarithmic gap penalty though much slower than sse2 version.")
   ("gi", po::value<int>(&iGI), "gap initialization penalty")
   ("gx", po::value<int>(&iGX), "gap extension penalty")
   ("mat", po::value<string>(), "file name for alignment scoring matrix")
   ;

   backend.add_options()
   ("nothreads","do not use multithreading. Useful if for some reason you need to reduce CPU utilization to just 1 core.")
   ;
   hidden.add_options()
   ("input-file1", po::value<string>(&infile1), "top sequence file.")
   ("input-file2", po::value<string>(&infile2), "bottom sequence file.")
   ("output-file", po::value<string>(&outfile)->default_value("-"), "output file. If '-' - stdout will be taken")
   ;

   po::options_description cmdline_options;
   cmdline_options.add(generic).add(settings).add(alignment).add(backend).add(hidden);

   po::options_description visible_options;
   visible_options.add(generic).add(settings).add(alignment).add(backend);

   po::positional_options_description p;
   p.add("input-file1", 1).add("input-file2", 1).add("output-file", 1);

   po::variables_map vm;
   po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
   po::notify(vm);

   if (vm.count("version")) {
      showVersion(argv[0]);
      exit(0);
   }
   if (vm.count("input-file1") == 0 || vm.count("input-file2") == 0) {
      usage(argv[0], visible_options);
      exit(0);
   }
   if (vm.count("debug")) {
      //DEBUG   = true;
   }
   if (vm.count("pgsw")) {
      aligner.setAlignmentMethod(PGSWAT);
   }
   if (vm.count("mon")) {
      aligner.setAlignmentMethod(MONARDO);
   }
   if (vm.count("sse2")) {
      aligner.setAlignmentMethod(SWSSE2);
   }
   if (vm.count("protein")) {
      BioAlignment::loadMatrix("/usr/local/lib/mat/BLOSUM62");
      aligner.setAlignmentMatrix("/usr/local/lib/mat/BLOSUM62");
   }
   if (vm.count("mat")) {
      BioAlignment::loadMatrix(vm["mat"].as<string> ().c_str());
      aligner.setAlignmentMatrix(vm["mat"].as<string> ().c_str());
   }
   if (vm.count("except")) {
      codonSubset = true;
   }
   if (vm.count("nothreads")) {
      useThreads = false;
   }

   if (iGI != 0) {
      aligner.setGapIni(iGI);
   }
   if (iGX != 0) {
      aligner.setGapExt(iGX);
   }

   IstreamWrapper is1(infile1.c_str());
   IstreamWrapper is2(infile2.c_str());
   OstreamWrapper os(outfile.c_str());
   //========================================================
   if (useThreads) {
      ThreadPool tp;
      worker_params params(is1, is2, os, aligner, offset, codonSubset, sharp);
      tp.initWorkers(thread_func, &params);
      tp.joinWorkers();
   }
   //Single threaded inmplementation
   else {
      long iScore = 0;
      BioAlignment aln;
      while (aln.getSeq().read(is1) >= 0 && aln.getCon().read(is2) >= 0) {
         aln.getCon().toupper();
         aln.getSeq().toupper();

         if (aligner.align(aln)) {
            if (sharp > 0) {
               aln.sharp(sharp);
               if (aln.getScore() == 0) {
                  continue;
               }
            }
            if (offset) {
               os.stream() << aln.getCon().getName() << "\t+" << offset - 1 << "\t" << aln.getSeq().getName() << "\t";
               if (codonSubset) {
                  getCDSStatX(aln.getCon(), aln.getSeq(), aln.getCon().getFrom() - 1, offset - 1).print(os);
               } else {
                  getCDSStat(aln.getCon(), aln.getSeq(), aln.getCon().getFrom() - 1, offset - 1).print(os);
               }
            }
            aln.print(os);
         } else {
            cerr << "No alignment between " << aln.getCon().getName() << " and " << aln.getSeq().getName() << endl;
         }
      }
   }
   //========================================================
}
