#include <math.h>
#include <string>
#include <vector>
#include <sstream>
#include "libmonardo.h"
#include "BioSequence.h"
//#include "SWATMat.h"

using namespace math;
using namespace std;

/** ======================================================== */
inline float gap_penalty(int l) {
#ifdef ALTWGHT
   if(l < _MAX_GAP_LENGTH && l >= 0)
   {
      static float precalc[_MAX_GAP_LENGTH];
      if(precalc[l] == 0)
      {
         precalc[l] = (l == 0 ? _GAP_OPENING_COST : _GAP_OPENING_COST + _GAP_COST_PER_LETTER * log((double)l));
      }
      return precalc[l];
   }
   return l == 0 ? _GAP_OPENING_COST : _GAP_OPENING_COST + _GAP_COST_PER_LETTER * log((double)l);
#else
   return l == 0 ? _GAP_OPENING_COST : _GAP_OPENING_COST + _GAP_COST_PER_LETTER * l;
#endif
}

/** ======================================================== */
float smith_local_score(BioSubSequence& av, BioSubSequence& bv) {
   string s1(av);
   string s2(bv);
   av = "";
   bv = "";
   size_t b1, e1, b2, e2;
   float score = smith_local_score(s1.begin(), s1.end(), s2.begin(), s2.end(), back_insert_iterator<string>(av), back_insert_iterator<string>(bv), b1, e1, b2, e2);
   av.setFrom(b1);
   av.setTo(e1);
   bv.setFrom(b2);
   bv.setTo(e2);
   return score;
}
/** ======================================================== */
void printMatrix(matrix<float> &mat, string& rowHead, string& colHead) {
   int x, y;
   fprintf(stderr, "%10c", ' ');
   for (x = 1; x < mat.ColNo(); x++) {
      fprintf(stderr, "%-c:%-3d", colHead[x - 1], x);
   }
   fprintf(stderr, "\n");
   for (y = 0; y < mat.RowNo(); y++) {
      if (y > 0) {
         fprintf(stderr, "%-c:%-3d", rowHead[y - 1], y);
      } else {
         fprintf(stderr, "%5c", ' ');
      }
      for (x = 0; x < mat.ColNo(); x++) {
         fprintf(stderr, "%-5d", int(10 * mat(y, x)));
      }
      fprintf(stderr, "\n");
   }
}

/** ======================================================== */
void printMatrix(matrix<trace> &mat, string& rowHead, string& colHead) {
   int x, y;
   fprintf(stderr, "%10c", ' ');
   for (x = 1; x < mat.ColNo(); x++) {
      fprintf(stderr, "%c:%-3d", colHead[x - 1], x);
   }
   fprintf(stderr, "\n");
   for (y = 0; y < mat.RowNo(); y++) {
      if (y > 0) {
         fprintf(stderr, "%c:%-3d", rowHead[y - 1], y);
      } else {
         fprintf(stderr, "%5c", ' ');
      }
      for (x = 0; x < mat.ColNo(); x++) {
         switch (mat(y, x).dir) {
         case UP:
            fprintf(stderr, "%-5d", mat(y, x).len);
            break;
         case OVER:
            fprintf(stderr, "%-5d", -mat(y, x).len);
            break;
         case DIAG:
            fprintf(stderr, "%-5c", '\\');
            break;
         case ZIP:
         default:
            fprintf(stderr, "%-5c", 'X');
         }
      }
      fprintf(stderr, "\n");
   }
}

