#ifndef __LOCAL__
#define __LOCAL__

#define ALTWGHT

#define _CHAR_MATCH_VALUE 1.0
//_UINT_MATCH_VALUE - is for alignment of numerical codes, not numerical values.
//For alignment of numerical values absolute value of difference should be used.
#define _UINT_MATCH_VALUE 6.0

#ifndef ALTWGHT
#define _MISMATCH_COST 0.91
#define _GAP_OPENING_COST 1.01
#define _GAP_COST_PER_LETTER 0.91
#else
#define _MISMATCH_COST 1.0
//#define _GAP_OPENING_COST 2.95
#define _GAP_OPENING_COST 2.55
#define _GAP_COST_PER_LETTER 1.90
#endif
#define _MAX_GAP_LENGTH 1500

enum direction {
   ZIP, DIAG, UP, OVER
};

struct trace {
   direction dir;
   short len;
   trace() :
      dir(ZIP), len(0) {
   }
   trace(int) :
      dir(ZIP), len(0) {
   }
};

extern int GAP;
#include "BioSequence.h"
#include "matrix.h"

using namespace math;

#include <map>
using namespace std;

float gap_penalty(int l);
float smith_local_score(BioSubSequence& av, BioSubSequence& bv);
void printMatrix(matrix<float> &mat, string& rowHead, string& colHead);
void printMatrix(matrix<trace> &mat, string& rowHead, string& colHead);

inline char GapVal(char) {
   return '-';
}
inline int GapVal(int) {
   return 0;
}
inline float GapVal(float) {
   return 0.0;
}
inline float dist(char a, char b) {
   return (a == b) ? _CHAR_MATCH_VALUE : -_MISMATCH_COST;
}
inline float dist(unsigned int a, unsigned int b) {
   return (a == b) ? _UINT_MATCH_VALUE : -_MISMATCH_COST;
}
inline float dist(int a, int b) {
   return _UINT_MATCH_VALUE / abs(a - b);
}

template<class BidirectionalIterator, class OutputIterator>
float smith_local_score(BidirectionalIterator beg1, BidirectionalIterator end1, BidirectionalIterator beg2, BidirectionalIterator end2, OutputIterator optr1, OutputIterator optr2, size_t &b1,
      size_t &e1, size_t &b2, size_t &e2) {
   const size_t size1 = distance(beg1, end1);
   const size_t size2 = distance(beg2, end2);

   matrix<float> hm(size2 + 1, size1 + 1);
   matrix<trace> tm(size2 + 1, size1 + 1);

   vector<int> overv(size1 + 1);
   vector<int> upv(size2 + 1);

   for (int j = 0; j < hm.RowNo(); j++) {
      upv[j] = 0;
      hm(j, 0) = 0.0;
      tm(j, 0).dir = ZIP;
      tm(j, 0).len = 0;
   }
   for (int i = 0; i < hm.ColNo(); i++) {
      overv[i] = 0;
      hm(0, i) = 0.0;
      tm(0, i).dir = ZIP;
      tm(0, i).len = 0;
   }

   int ii = 0;
   int jj = 0;
   float hmax = 0;
   int i, j;
   BidirectionalIterator ptr1, ptr2;
   BidirectionalIterator mptr1, mptr2;
   for (i = 1, ptr1 = beg1; ptr1 != end1; ++i, ++ptr1) {
      for (j = 1, ptr2 = beg2; ptr2 != end2; ++j, ++ptr2) {
         float hh = 0.0;
         hm(j, i) = 0.0;
         tm(j, i).dir = ZIP;
         tm(j, i).len = j;
         float m = hm(j - 1, i - 1) + dist(*ptr1, *ptr2);
         if (m > 0.0) {
            hm(j, i) = hh = m;
            tm(j, i).dir = DIAG;
            tm(j, i).len = 1;
         }
         int k = upv[j];
         int l = i - k;
         float hup = hm(j, k) - gap_penalty(l);
         if (hup > hh) {
            hm(j, i) = hh = hup;
            tm(j, i).dir = UP;
            tm(j, i).len = l;
         }
         k = overv[i];
         l = j - k;
         float hover = hm(k, i) - gap_penalty(l);
         if (hover > hh) {
            hm(j, i) = hh = hover;
            tm(j, i).dir = OVER;
            tm(j, i).len = l;
         }
         if (hh > hmax) {
            hmax = hh;
            ii = i - 1;
            jj = j - 1;
            mptr1 = ptr1;
            mptr2 = ptr2;
         }
         if (hh == m || hh == 0.0) {
            if (hh >= hup + gap_penalty(0)) {
               upv[j] = i;
            }
            if (hh >= hover + gap_penalty(0)) {
               overv[i] = j;
            }
         }
      }
   }

#ifdef DEBUG
   printMatrix(tm, bv, av);
#endif

   /* NEW TRACING */
   //fprintf(stderr, "av.size=%d\n", av.size());
   //fprintf(stderr, "bv.size=%d\n", bv.size());
   int idx;
   e1 = ii + 1;
   e2 = jj + 1;
   vector<typename iterator_traits<BidirectionalIterator>::value_type> aav;
   vector<typename iterator_traits<BidirectionalIterator>::value_type> bbv;
   while (ii >= 0 && jj >= 0) {
      trace &tt = tm(jj + 1, ii + 1);
      int len = tt.len;
      switch (tt.dir) {
      case ZIP:
         while (ii >= 0) {
            aav.push_back(*mptr1);
            --ii;
            --mptr1;
            bbv.push_back(GapVal(*mptr2));
         }
         while (jj >= 0) {
            aav.push_back(GapVal(*mptr1));
            bbv.push_back(*mptr2);
            --jj;
            --mptr2;
         }
         break;
      case DIAG:
         while (len--) {
            aav.push_back(*mptr1);
            --ii;
            --mptr1;
            bbv.push_back(*mptr2);
            --jj;
            --mptr2;
         }
         idx = aav.size();
         b1 = ii + 2;
         b2 = jj + 2;
         break;
      case UP:
         while (len--) {
            aav.push_back(*mptr1);
            --ii;
            --mptr1;
            bbv.push_back(GapVal(*mptr2));
         }
         break;

      case OVER:
         while (len--) {
            aav.push_back(GapVal(*mptr1));
            bbv.push_back(*mptr2);
            --jj;
            --mptr2;
         }
         break;
      default:
         throw new domain_error("Bad DIR field in alignment traceback");
      }
   }
   while (--idx >= 0) {
      *optr1 = aav[idx];
      *optr2 = bbv[idx];
      ++optr1;
      ++optr2;
   }
   return hmax;
}

#endif

